package com.rubin.connector;

import com.rubin.LifeCycle;
import com.rubin.engine.Engine;
import com.rubin.reader.ServerXmlReader;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.concurrent.*;

/**
 * Created by RubinChu on 2021/5/2 11:29
 */
public class Connector implements LifeCycle {

    private Engine engine;

    private Integer port = 8080;

    private ServerSocket serverSocket;

    /**
     * 初始化
     */
    @Override
    public void init() {
        initConnector();
        this.engine = new Engine();
        engine.init();
    }

    /**
     * 初始化Connector
     */
    private void initConnector() {
        this.port = getPort();
        try {
            this.serverSocket = new ServerSocket(port);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 获取配置的port
     *
     * @return
     */
    private Integer getPort() {
        String portKey = "Connector.port";
        ServerXmlReader serverXmlReader = new ServerXmlReader();
        String portStr = serverXmlReader.read(portKey);
        return portStr == null || "".equals(portStr) ? port : Integer.valueOf(portStr);
    }

    /**
     * 启动
     */
    @Override
    public void start() {
        engine.start();

        // 定义一个线程池
        int corePoolSize = 10;
        int maximumPoolSize =50;
        long keepAliveTime = 100L;
        TimeUnit unit = TimeUnit.SECONDS;
        BlockingQueue<Runnable> workQueue = new ArrayBlockingQueue<>(50);
        ThreadFactory threadFactory = Executors.defaultThreadFactory();
        RejectedExecutionHandler handler = new ThreadPoolExecutor.AbortPolicy();
        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(
                corePoolSize,
                maximumPoolSize,
                keepAliveTime,
                unit,
                workQueue,
                threadFactory,
                handler
        );
        System.out.println("Server start on port: " + port);
        while (true) {
            try {
                Socket socket = serverSocket.accept();
                threadPoolExecutor.execute(new RequestProcessor(socket, engine));
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 销毁
     */
    @Override
    public void destroy() {
        engine.destroy();
    }

}
