package com.rubin.host;

import com.rubin.LifeCycle;
import com.rubin.mapper.Mapper;
import com.rubin.reader.ServerXmlReader;
import com.rubin.servlet.Servlet;
import com.rubin.utils.StaticResourceUtil;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Created by RubinChu on 2021/5/2 11:38
 */
public class Host implements LifeCycle {

    private String name;

    private String appBase;

    private List<Mapper> mappers;

    /**
     * 初始化
     */
    @Override
    public void init() {
        ServerXmlReader serverXmlReader = new ServerXmlReader();
        this.name = serverXmlReader.read("Host.name");
        this.appBase = serverXmlReader.read("Host.appBase");
        initMappers();
    }

    /**
     * 初始化映射器
     */
    private void initMappers() {
        mappers = new ArrayList<>();
        File baseFile = new File(appBase);
        if (!baseFile.exists()) {
            appBase = StaticResourceUtil.getAbsolutePath(appBase);
            baseFile = new File(appBase);
        }
        if (baseFile.exists() && baseFile.isDirectory()) {
            Arrays.stream(baseFile.listFiles()).forEach(file -> {
                if (file.isDirectory()) {
                    Mapper mapper = new Mapper(file);
                    mappers.add(mapper);
                }
            });
        } else {
            throw new RuntimeException("the appBase : " + appBase + "is not available");
        }
    }

    /**
     * 启动
     */
    @Override
    public void start() {

    }

    /**
     * 销毁
     */
    @Override
    public void destroy() {

    }

    public Servlet selectServlet(String url) {
        for (Mapper mapper : mappers) {
            Servlet servlet = mapper.selectServlet(url);
            if (servlet != null) {
                return servlet;
            }
        }
        return null;
    }

}
