package com.rubin.mapper;

import com.rubin.classloader.ServletClassLoader;
import com.rubin.reader.WebXmlReader;
import com.rubin.servlet.Servlet;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by RubinChu on 2021/5/2 13:42
 */
public class Mapper {

    private Map<String, MappedContent> mappedContentMap = new HashMap<>();

    public Mapper(File baseFolder) {
        String contentName = baseFolder.getName();
        MappedContent mappedContent = new MappedContent(contentName, baseFolder);
        mappedContentMap.put(contentName, mappedContent);
    }

    public Servlet selectServlet(String url) {
        if (url.startsWith("/")) {
            url = url.substring(1);
        }
        if (url.indexOf("/") == -1) {
            return null;
        }
        String contentName = url.substring(0, url.indexOf("/"));
        MappedContent mappedContent = mappedContentMap.get(contentName);
        if (mappedContent == null) {
            return null;
        }
        String uri = url.substring(contentName.length());
        return mappedContent.selectServlet(uri);
    }

    private static class MappedContent {

        private String contentName;

        private File baseFolder;

        private Map<String, Servlet> servletMap;

        public MappedContent(String contentName, File baseFolder) {
            this.contentName = contentName;
            this.baseFolder = baseFolder;
            this.servletMap = new HashMap<>();
            initServlets();
        }

        private void initServlets() {
            File webXml = new File(baseFolder.getPath() + File.separator + "web.xml");
            if (webXml.exists() && webXml.isFile()) {
                WebXmlReader webXmlReader = new WebXmlReader(webXml);
                webXmlReader.read().forEach(entry -> {
                    servletMap.put(entry.getKey(), loadServletInstance(entry.getValue()));
                });
            }
        }

        private Servlet loadServletInstance(String value) {
            try {
                Class clazz = new ServletClassLoader(baseFolder.getPath()).findClass(value);
                return (Servlet) clazz.getDeclaredConstructor().newInstance();
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (InstantiationException e) {
                e.printStackTrace();
            } catch (NoSuchMethodException e) {
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                e.printStackTrace();
            }
            throw new RuntimeException("find class " + value + "error");
        }

        public Servlet selectServlet(String uri) {
            return servletMap.get(uri);
        }

    }

}
