package com.rubin.reader;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.dom4j.tree.DefaultAttribute;

import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Created by RubinChu on 2021/5/2 12:58
 */
public class ServerXmlReader {

    private Map<String, String> config = new HashMap<>();

    public ServerXmlReader() {
        InputStream resourceAsStream = ServerXmlReader.class.getClassLoader().getResourceAsStream("server.xml");
        SAXReader saxReader = new SAXReader();
        // 读取配置信息，封装进缓存中
        try {
            Document document = saxReader.read(resourceAsStream);
            Element rootElement = document.getRootElement();
            parseElement(rootElement);
        } catch (DocumentException e) {
            e.printStackTrace();
        }
    }

    /**
     * 解析根元素下的所有元素
     *
     * @param element
     */
    private void parseElement(Element element) {
        if (element == null) {
            return;
        }
        String elementName = element.getName();
        Iterator iterator = element.attributeIterator();
        while (iterator.hasNext()) {
            DefaultAttribute defaultAttribute = (DefaultAttribute) iterator.next();
            String key = elementName + "." + defaultAttribute.getName();
            String value = defaultAttribute.getValue();
            config.put(key, value);
        }
        List childrenElements = element.elements();
        if (childrenElements != null && childrenElements.size() > 0) {
            for (Object childrenElement : childrenElements) {
                parseElement((Element) childrenElement);
            }
        }
    }

    /**
     * 获取配置信息 key=标签名称 + 属性值key
     *
     * @param key
     * @return
     */
    public String read(String key) {
        return config.get(key);
    }

}
